<?php

/**
 * Remote controlling the Raspberry Pi over WWW
 */

date_default_timezone_set('UTC');

define('PSK',        'r6rlkzS90glk');
define('LOG_FILE',   '/tmp/rpi_control.log');
define('QUEUE_FILE', '/tmp/rpi_control.queue');

// valid commands for the RPi
$COMMANDS = ['LED_ON', 'LED_OFF', 'SHUTDOWN', 'EXIT'];

if (isset($_POST['nextCommand'])) {
	$queue = @file(QUEUE_FILE, FILE_SKIP_EMPTY_LINES);
	if (!is_array($queue)) {
		$queue = [];
	}

	$command = trim($_POST['nextCommand']);
	if (!in_array($command, $COMMANDS)) {
		logAction('ERROR: Invalid command "'.$command.'" provided');
	} else {
		$queue[] = $command . PHP_EOL;
		if (file_put_contents(QUEUE_FILE, $queue)) {
			logAction('INFO: Added new command "' . $command . '" to queue');
		} else {
			logAction('ERROR: Cannot write new command "' . $command . '" to queue');
		}
	}
}

// IoT device request the next command
if (isset($_GET['request'])) {
	authorize();
	$queue = @file(QUEUE_FILE, FILE_SKIP_EMPTY_LINES);
	if (!is_array($queue) || count($queue) == 0) {
		exit();
	}
	$command = trim(array_shift($queue));
	if (file_put_contents(QUEUE_FILE, $queue) !== false) {
		logAction('INFO: Delivered command "'.$command.'"');
		exit($command);
	} else {
		logAction('ERROR: Cannot remove command "'.$command.'" from queue', true);
	}
}

function authorize() {
	if (!isset($_GET['psk']) || $_GET['psk'] != PSK) {
		logAction('ERROR: Unauthorized request', true);
	}
}

function logAction ($message, $doExit = false) {
	$ipAddr = isset($_SERVER['REMOTE_ADDR'])?$_SERVER['REMOTE_ADDR']:'?';
	$message = date('c', time()) . ' - ' . $message . ' (IP: ' . $ipAddr . ')' . PHP_EOL;
	error_log($message, 3, LOG_FILE);
	if ($doExit) {
	    exit();
    }
}

?>
<html>
	<head>
		<meta charset="UTF-8"/>
		<title>Raspberry Pi Control</title>
		<style type="text/css">
			.column {
				float: left;
				margin-right: 25px;
				background-color: #F0F0F0;
				padding: 0 10px 10px 10px;
				border: 1px solid #D0D0D0;
			}
		</style>
	</head>
	<body>
		<h1>Raspberry Pi Control</h1>
		<div class="column">
			<h2>Next command</h2>
			<form method="post" action="">
				<select id="nextCommand" name="nextCommand">
					<option>-- please choose --</option>
					<?php
					foreach ($COMMANDS as $command) {
						print '<option>'.$command.'</option>';
					}
					?>
				</select>
				<br/>
				<input type="submit" value="send"/>
			</form>
		</div>
		<div class="column">
			<h2>Queue</h2>
			<ul>
				<?php
				$queue = @file(QUEUE_FILE, FILE_SKIP_EMPTY_LINES);
				if (is_array($queue)) {
					foreach ($queue as $command) {
						print '<li>' . $command . '</li>';
					}
				}
				?>
			</ul>
		</div>
	</body>
</html>